@@ -35,7 +35,7 @@ import unittest
 from textwrap import dedent
 from nose.tools import assert_equal, assert_almost_equal, raises
 
-from affine import Affine
+from affine import Affine, EPSILON
 
 
 def seq_almost_equal(t1, t2, error=0.00001):
@@ -123,9 +123,9 @@ class PyAffineTestCase(unittest.TestCase):
         assert isinstance(trans, Affine)
         assert_equal(
             tuple(trans),
-            (1, 0,  2,
+            (1, 0, 2,
              0, 1, -5,
-             0, 0,  1))
+             0, 0, 1))
 
     def test_scale_constructor(self):
         scale = Affine.scale(5)
@@ -139,8 +139,8 @@ class PyAffineTestCase(unittest.TestCase):
         assert_equal(
             tuple(scale),
             (-1, 0, 0,
-              0, 2, 0,
-              0, 0, 1))
+             0, 2, 0,
+             0, 0, 1))
         assert_equal(tuple(Affine.scale(1)), tuple(Affine.identity()))
 
     def test_shear_constructor(self):
@@ -158,8 +158,8 @@ class PyAffineTestCase(unittest.TestCase):
         seq_almost_equal(
             tuple(shear),
             (1, mx, 0,
-            my,  1, 0,
-             0,  0, 1))
+             my, 1, 0,
+             0, 0, 1))
         shear = Affine.shear(y_angle=45)
         seq_almost_equal(
             tuple(shear),
@@ -174,16 +174,16 @@ class PyAffineTestCase(unittest.TestCase):
         s, c = math.sin(r), math.cos(r)
         assert_equal(
             tuple(rot),
-            (c, s, 0,
-            -s, c, 0,
+            (c, -s, 0,
+             s, c, 0,
              0, 0, 1))
         rot = Affine.rotation(337)
         r = math.radians(337)
         s, c = math.sin(r), math.cos(r)
         seq_almost_equal(
             tuple(rot),
-            (c, s, 0,
-            -s, c, 0,
+            (c, -s, 0,
+             s, c, 0,
              0, 0, 1))
         assert_equal(tuple(Affine.rotation(0)), tuple(Affine.identity()))
 
@@ -195,29 +195,29 @@ class PyAffineTestCase(unittest.TestCase):
              0, 0, 1))
         assert_equal(
             tuple(Affine.rotation(90)),
-            (0, 1, 0,
-            -1, 0, 0,
+            (0, -1, 0,
+             1, 0, 0,
              0, 0, 1))
         assert_equal(
             tuple(Affine.rotation(180)),
-            (-1,  0, 0,
-              0, -1, 0,
-              0,  0, 1))
+            (-1, 0, 0,
+             0, -1, 0,
+             0, 0, 1))
         assert_equal(
             tuple(Affine.rotation(-180)),
-            (-1,  0, 0,
-              0, -1, 0,
-              0,  0, 1))
+            (-1, 0, 0,
+             0, -1, 0,
+             0, 0, 1))
         assert_equal(
             tuple(Affine.rotation(270)),
-            (0, -1, 0,
-             1,  0, 0,
-             0,  0, 1))
+            (0, 1, 0,
+             -1, 0, 0,
+             0, 0, 1))
         assert_equal(
             tuple(Affine.rotation(-90)),
-            (0, -1, 0,
-             1,  0, 0,
-             0,  0, 1))
+            (0, 1, 0,
+             -1, 0, 0,
+             0, 0, 1))
         assert_equal(
             tuple(Affine.rotation(360)),
             (1, 0, 0,
@@ -225,25 +225,25 @@ class PyAffineTestCase(unittest.TestCase):
              0, 0, 1))
         assert_equal(
             tuple(Affine.rotation(450)),
-            (0, 1, 0,
-            -1, 0, 0,
+            (0, -1, 0,
+             1, 0, 0,
              0, 0, 1))
         assert_equal(
             tuple(Affine.rotation(-450)),
-            (0, -1, 0,
-             1,  0, 0,
-             0,  0, 1))
+            (0, 1, 0,
+             -1, 0, 0,
+             0, 0, 1))
 
     def test_rotation_constructor_with_pivot(self):
         assert_equal(tuple(Affine.rotation(60)),
-            tuple(Affine.rotation(60, pivot=(0, 0))))
+                     tuple(Affine.rotation(60, pivot=(0, 0))))
         rot = Affine.rotation(27, pivot=(2, -4))
         r = math.radians(27)
         s, c = math.sin(r), math.cos(r)
         assert_equal(
             tuple(rot),
-            (c, s, 2 - 2 * c - 4 * s,
-            -s, c, -4 - 2 * s + 4 * c,
+            (c, -s, 2 - 2 * c - 4 * s,
+             s, c, -4 - 2 * s + 4 * c,
              0, 0, 1))
         assert_equal(tuple(Affine.rotation(0, (-3, 2))),
                      tuple(Affine.identity()))
@@ -288,7 +288,6 @@ class PyAffineTestCase(unittest.TestCase):
         assert not Affine.shear(4, -1).is_orthonormal
 
     def test_is_degenerate(self):
-        from affine import EPSILON
         assert not Affine.identity().is_degenerate
         assert not Affine.translation(2, -1).is_degenerate
         assert not Affine.shear(0, -22.5).is_degenerate
@@ -299,7 +298,6 @@ class PyAffineTestCase(unittest.TestCase):
         assert Affine.scale(0, 300).is_degenerate
         assert Affine.scale(0).is_degenerate
         assert Affine.scale(0).is_degenerate
-        assert Affine.scale(EPSILON).is_degenerate
 
     def test_column_vectors(self):
         a, b, c = Affine(2, 3, 4, 5, 6, 7).column_vectors
@@ -311,8 +309,7 @@ class PyAffineTestCase(unittest.TestCase):
         assert_equal(c, (4, 7))
 
     def test_almost_equals(self):
-        from affine import EPSILON
-        assert EPSILON != 0, EPSILON
+        EPSILON = 1e-5
         E = EPSILON * 0.5
         t = Affine(1.0, E, 0, -E, 1.0 + E, E)
         assert t.almost_equals(Affine.identity())
@@ -323,6 +320,18 @@ class PyAffineTestCase(unittest.TestCase):
         assert not Affine.identity().almost_equals(t)
         assert t.almost_equals(t)
 
+    def test_almost_equals_2(self):
+        EPSILON = 1e-10
+        E = EPSILON * 0.5
+        t = Affine(1.0, E, 0, -E, 1.0 + E, E)
+        assert t.almost_equals(Affine.identity(), precision=EPSILON)
+        assert Affine.identity().almost_equals(t, precision=EPSILON)
+        assert t.almost_equals(t, precision=EPSILON)
+        t = Affine(1.0, 0, 0, -EPSILON, 1.0, 0)
+        assert not t.almost_equals(Affine.identity(), precision=EPSILON)
+        assert not Affine.identity().almost_equals(t, precision=EPSILON)
+        assert t.almost_equals(t, precision=EPSILON)
+
     def test_equality(self):
         t1 = Affine(1, 2, 3, 4, 5, 6)
         t2 = Affine(6, 5, 4, 3, 2, 1)
@@ -398,25 +407,13 @@ class PyAffineTestCase(unittest.TestCase):
         seq_almost_equal(
             ~Affine.rotation(-33.3), Affine.rotation(33.3))
         t = Affine(1, 2, 3, 4, 5, 6)
-        seq_almost_equal(~t * t,  Affine.identity())
+        seq_almost_equal(~t * t, Affine.identity())
 
     def test_cant_invert_degenerate(self):
         from affine import TransformNotInvertibleError
         t = Affine.scale(0)
         self.assertRaises(TransformNotInvertibleError, lambda: ~t)
 
-    def test_set_epsilon(self):
-        import affine
-
-        old_epsilon = affine.EPSILON
-
-        try:
-            affine.set_epsilon(123)
-            assert_equal(123, affine.EPSILON)
-            assert_equal(123 * 123, affine.EPSILON2)
-        finally:
-            affine.set_epsilon(old_epsilon)
-
     @raises(TypeError)
     def test_bad_type_world(self):
         from affine import loadsw
@@ -460,6 +457,9 @@ class PyAffineTestCase(unittest.TestCase):
         self.assertTrue(a1.almost_equals(a2))
 
 
+# We're using pytest for tests added after 1.0 and don't need unittest
+# test case classes.
+
 def test_gdal():
     t = Affine.from_gdal(-237481.5, 425.0, 0.0, 237536.4, 0.0, -425.0)
     assert t.c == t.xoff == -237481.5
@@ -472,6 +472,51 @@ def test_gdal():
     assert t.to_gdal() == (-237481.5, 425.0, 0.0, 237536.4, 0.0, -425.0)
 
 
+def test_imul_number():
+    t = Affine(1, 2, 3, 4, 5, 6)
+    try:
+        t *= 2.0
+    except TypeError:
+        assert True
+
+
+def test_mul_tuple():
+    t = Affine(1, 2, 3, 4, 5, 6)
+    t * (2.0, 2.0)
+
+
+def test_rmul_tuple():
+    t = Affine(1, 2, 3, 4, 5, 6)
+    (2.0, 2.0) * t
+
+
+def test_transform_precision():
+    t = Affine.rotation(45.0)
+    assert t.precision == EPSILON
+    t.precision = 1e-10
+    assert t.precision == 1e-10
+    assert Affine.rotation(0.0).precision == EPSILON
+
+
+def test_associative():
+    point = (12, 5)
+    trans = Affine.translation(-10., -5.)
+    rot90 = Affine.rotation(90.)
+    result1 = rot90 * (trans * point)
+    result2 = (rot90 * trans) * point
+    seq_almost_equal(result1, (0., 2.))
+    seq_almost_equal(result1, result2)
+
+
+def test_roundtrip():
+    point = (12, 5)
+    trans = Affine.translation(3, 4)
+    rot37 = Affine.rotation(37.)
+    point_prime = (trans * rot37) * point
+    roundtrip_point = ~(trans * rot37) * point_prime
+    seq_almost_equal(point, roundtrip_point)
+
+
 if __name__ == '__main__':
     unittest.main()
 
